
/*
 * File   : atvme.c
 *
 * Purpose: Library for ProVme Datentechniks Vme-Interface ATVME
 *          interfacing Linux loadable module ...
 *
 * $Revision: 0.23 $
 * $Date: 1999/02/17 13:33:51 $
 * $Id: atvme.c,v 0.23 1999/02/17 13:33:51 dirk Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fcntl.h>
#include <sys/ioctl.h>
#include <time.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <asm/page.h>
#include <malloc.h>

#include <linux/atvme.h>

#include "atvme.h"



static int Atvme_fd = 0;
static void *Atvme_allocated_page = NULL;
static int Atvme_int_enabled = 0;

int Atvme_fpos_tied = 1;  /* read()/write() will move fpos */

static char Atvme_version_string[] =
  "Atvme Linux Version $Revision: 0.23 $ $Date: 1999/02/17 13:33:51 $";
 
/* -- use this function if device is opened by an external routine --------- */

void AtvmeSetFD ( int fd ) { Atvme_fd = fd; }               /* tow           */



/* ------------------------------------------------------------------------- */

char *AtvmeGetVersionString ( void )
 {                                                          /* thjm 10.11.98 */
  return Atvme_version_string;
}

/* ------------------------------------------------------------------------- */
/*                                                                           */
/* INIT functions   for AT_VME and VMEbus                                    */
/*                                                                           */
/*                                                                           */
/* changed by DM: returns fd                                                 */

int AtvmeOpen ( void )
                                                /* Inits AT_VME */
 {
  char buffer[1024];
  int loop = 5;

#if 0
  printf ( "AtvmeInit: MemBase= 0x%x, IoBase= 0x%x\n", 
            AtvmeMemBase, AtvmeIOBase );
#endif
  
  if ( Atvme_fd == 0 )   /* nobody opened it before ... */
   {
    /* on slow systems, module autoloading is too slow !? */
    
    while ( loop && (Atvme_fd <= 0) )
     {
      Atvme_fd = open ( "/dev/atvme", O_RDWR );
      loop--;
      if ( Atvme_fd < 0 )
        sleep ( 1 );
    }
    
    if ( Atvme_fd < 0 )
     {
      Atvme_fd = 0;
      perror ( "open" );

      return -1;
    }
  
    ioctl ( Atvme_fd, ATVME_GET_DRIVER_INFO, (unsigned long)buffer );
    puts ( buffer );
      
    ioctl ( Atvme_fd, ATVME_INITIALIZE, 0 ); 
    ioctl ( Atvme_fd, ATVME_UNTIE_FPOS, 0 );
    Atvme_fpos_tied = 0;
 
    AtvmeResetVme();
  }
  
  return Atvme_fd;
}

/* ------------------------------------------------------------------------- */

int AtvmeGetFd ( void )
 {                                                          /* thjm 25.11.98 */
  return Atvme_fd;
}

/* ------------------------------------------------------------------------- */

int AtvmeCheckConnection ( void )
                               /* checks the proper connection of AT and VME */
 {
  int ret;

  if ( Atvme_fd != 0 )
   {
    if ( ioctl ( Atvme_fd, ATVME_CHECK_CONNECTION, &ret ) < 0 )
     {
      perror ( "ioctl(CHECK_CONNECTION)" );
      ret = 0;
    }
  }
  else
    ret = 0;
    
  return ( ret ); 
}

/* ------------------------------------------------------------------------- */

int AtvmeShutdown ( void )
                                           /* deinitializes AT_VME Interface */
 {
  int ret;
    
  if ( Atvme_fd != 0 )
   {
    ret = close ( Atvme_fd );
    Atvme_fd = 0;

    if ( !Atvme_fpos_tied )
     {
      ioctl ( Atvme_fd, ATVME_TIE_FPOS, 0 );
      Atvme_fpos_tied = 1;
    }
  }
  else
    ret = -1;
  
  if ( Atvme_allocated_page ) free ( Atvme_allocated_page );

  return ( ret );
}

/* ------------------------------------------------------------------------- */

int AtvmeResetVme ( void )
                                             /* generates SYSRESET on VMEbus */
 {
  if ( Atvme_fd != 0 )
    return ( ioctl ( Atvme_fd, ATVME_SYSRESET, 0 ) );
  else
    return ( -1 );
}

/* ------------------------------------------------------------------------- */
/* support for reading/writing VME bus                                       */
/*                                                                           */

/* ----- DM 5.10.98 -------------------------------------------------------- */

int AtvmeSetMode ( unsigned int mask )
 {
  static unsigned int old_mask = 0;
  AtvmeAD ad;
  
  if ( mask != old_mask )
   {
    old_mask = mask;
    
    ad.addr_type = mask & ATVME_A_MASK;
    ad.data_type = mask & ATVME_D_MASK;
    ad.acc_type  = mask & ( ATVME_ACC_MASK | ATVME_PRIV_MASK );
  }
  else
    return 0;
  
  return ioctl ( Atvme_fd, ATVME_SET_AM, (unsigned long)&ad );
}

/* ----- DM 5.10.98 -------------------------------------------------------- */

int AtvmeSeek ( unsigned int addr, int mode )
 {
  return lseek ( Atvme_fd, addr, mode );
}

/* ----- DM 5.10.98 -------------------------------------------------------- */

int AtvmeRead ( char *buf, int len )
 {
  return read ( Atvme_fd, buf, len );
}

/* ----- DM 5.10.98 -------------------------------------------------------- */

void AtvmeWrite ( char *buf, int len )
 {
  write ( Atvme_fd, buf, len );
}

/* ----- DM 19.11.98 -------------------------------------------------------- */

int AtvmeCheckError(void)
{
  int ret;
  ioctl ( Atvme_fd, ATVME_ERROR_CHECK, &ret);
  return ret;
}

/* ------------------------------------------------------------------------- */
/* interrupt stuff ...                                                       */
/*                                                                           */

/* ----- DM 8.10.98 -------------------------------------------------------- */

IntAckStruct *AtvmeGetNextPendingInt ( void )
/* gets the next interrupt from the pendingInts queue */
/* the struct contains the int vector and a timelabel */
 {
  IntAckStruct *ret = NEW(IntAckStruct);

  if ( ioctl ( Atvme_fd, ATVME_GET_INTERRUPT, ret ) < 0 )
   {
    free ( ret );
    return NULL;
  }

  return ret; 
}

/* ----- DM 18.11.98 -------------------------------------------------------- */

int AtvmePostInterrupt ( AtvmeAckType at, unsigned int rora_reg, int vec , unsigned int acc_mask)
{
  IntAckStruct ack;
  ack.ack_type = at;
  if (at != AtvmeRoak) {
    ack.rora_reg = rora_reg;
    ack.acc_mask = acc_mask;
  } else {
    ack.rora_reg = 0x0;
    ack.acc_mask = 0x0;
  } 
  ack.int_vec = vec;
  ack.int_tl = 0;
  return ioctl(Atvme_fd, ATVME_POST_INTERRUPT, &ack);
}

/* ----- DM 20.11.98 -------------------------------------------------------- */

int AtvmeRemoveInterrupt ( int vec )
{
  return ioctl(Atvme_fd, ATVME_POST_INTERRUPT, vec);
}

/* ----- DM 7.10.98 -------------------------------------------------------- */

int AtvmeEnableInterrupt ( void )
/* only VMEbus interrupts */
 {
  if (Atvme_int_enabled) return 0;
  Atvme_int_enabled = 1;
/* no check for success, 'cos ioctl() returns only 0 here */
  return ioctl ( Atvme_fd, ATVME_ENABLE_INTERRUPT ,0) ;
}

/* ----- DM 9.10.98 -------------------------------------------------------- */

int AtvmeDisableInterrupt ( void )
 {
  if ( !Atvme_int_enabled ) return 0;
  Atvme_int_enabled = 0 ;
  return ioctl ( Atvme_fd, ATVME_DISABLE_INTERRUPT, 0 );
}

/* ----- DM 7.10.98 -------------------------------------------------------- */

/*
 * Note: Please see into the drivers source code
 *       which type of file descriptor is used for select !
 */
 
int AtvmeSelect ( struct timeval *timeout )
 {
  fd_set read_fds;

  FD_ZERO ( &read_fds );
  FD_SET  ( Atvme_fd, &read_fds );
  
  return select ( Atvme_fd + 1, &read_fds, NULL, NULL, timeout );
}

/* ----- DM 25.1.99 -------------------------------------------------------- */

int AtvmeSignal ( void (*sig_handler)(int) )
{
  pid_t pid;
  struct sigaction sa;
  
  if ( sig_handler == SIG_IGN || sig_handler == SIG_DFL )
   {
    sig_handler = SIG_IGN;
    pid = 0;
  }
  else
    pid = getpid();

  sa.sa_handler = sig_handler;
  sa.sa_mask = 0x0;
  sa.sa_flags = SA_RESTART;
  sa.sa_restorer = NULL; 
  sigaction ( SIGUSR1, &sa, NULL );

  return ioctl ( Atvme_fd, ATVME_ATTACH_SIGNAL, pid );
}

/* ----- DM 30.11.98 ------------------------------------------------------- */

int AtvmeUntieFpos ( void )
 {
  if ( Atvme_fpos_tied )
   {
    Atvme_fpos_tied = 0;
    return ioctl ( Atvme_fd, ATVME_UNTIE_FPOS, 0 );
  }
  else
    return 0;
}

int AtvmeTieFpos ( void )
 {
  if ( Atvme_fpos_tied )
    return 0;
  else
   {
    Atvme_fpos_tied = 1;
    return ioctl ( Atvme_fd, ATVME_TIE_FPOS, 0 );
  }
}

/* ------------------------------------------------------------------------- */
/* mmap() stuff ...                                                          */
/*                                                                           */

/* ----- DM 9.10.98 -------------------------------------------------------- */

int AtvmeSet64KPage ( unsigned int addr )
 {
  return ioctl ( Atvme_fd, ATVME_SET_64K_PAGE, (unsigned long)addr );
}

/* ----- DM 8.10.98 -------------------------------------------------------- */

void *AtvmeMMap ( void )
 {
  char *ret;

  Atvme_allocated_page = valloc ( ATVME_SIZE );
  if ( !Atvme_allocated_page )
   {
    printf ( "AtvmeMMap(): no memory!\n" );
    return NULL;
  }

  ret = (char *)Atvme_allocated_page;

  ret = (unsigned char *)mmap ( (void *)ret, ATVME_SIZE, PROT_READ | PROT_WRITE,
                              MAP_SHARED | MAP_FIXED, Atvme_fd, ATVME_CSR );
  if ( (long)ret < 0L )
   {
    free ( Atvme_allocated_page );
    Atvme_allocated_page = NULL;
    return NULL;
  }

  return ret;
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
