
/*
 * File   : atvme.h
 *
 * Purpose: Defines for AT_VME Interface library
 *          Here interfacing the Linux driver
 *
 * $Revision: 0.16 $
 * $Date: 1999/02/17 13:33:54 $
 */

#ifndef _atvme_h_
#define _atvme_h_

#include <linux/atvme.h>
#include <linux/vme.h>
#include <signal.h>
#include <sys/time.h>

#ifndef NEW
# define NEW(t) (t *)malloc(sizeof(t))
#endif


typedef struct _VmeDesc
 {
  volatile void *addr;            /* pointer to the location */
  int acc;                        /* Vme access modes (not AM bits !!) */

} VmeDesc;

/* --- prototypes to the library --- */

/*
 * File: atvme.c
 */

__BEGIN_DECLS

extern void AtvmeSetFD ( int fd );
extern char *AtvmeGetVersionString ( void );

extern int AtvmeOpen ( void );            /* inits Atvme to ATVME_AM_DEFAULT */
extern int AtvmeGetFd ( void );               /* returns the file descriptor */
extern int AtvmeCheckConnection ( void );       /* check connection AT - VME */
extern int AtvmeShutdown ( void );                /* deinits Atvme interface */
extern int AtvmeResetVme ( void );           /* generates SYSRESET on VMEbus */
extern int AtvmeSetMode ( unsigned int am );
extern int AtvmeSeek ( unsigned int, int );
extern int AtvmeRead ( char *uf, int );
extern void AtvmeWrite ( char *buf, int );

/* --- for interrupts --- */

extern int AtvmeSelect ( struct timeval *tv );
extern int AtvmeSignal ( void (*)(int) );
extern int AtvmeEnableInterrupt ( void );
extern int AtvmeDisableInterrupt ( void );
extern int AtvmePostInterrupt ( AtvmeAckType ack_type, 
           unsigned int ack_address, int int_vector, unsigned int ack_access );
extern int AtvmeRemoveInterrupt ( int int_vector );
extern IntAckStruct *AtvmeGetNextPendingInt ( void );

/* --- mmap() stuff --- */

extern void *AtvmeMMap ( void );
extern int AtvmeSet64KPage ( unsigned int );

extern int Atvme_fpos_tied;
extern int AtvmeTieFpos ( void );
extern int AtvmeUntieFpos ( void );

/*
 * File: atvme_simple.c
 */
 
extern unsigned char VmeReadChar ( VmeDesc *vme );
extern void VmeWriteChar ( VmeDesc *vme, unsigned char data );
extern unsigned short VmeReadShort ( VmeDesc *vme );
extern void VmeWriteShort ( VmeDesc *vme, unsigned short data );

extern void VmeFifoReadShort ( VmeDesc *vme, 
                         unsigned short *data, unsigned int count );
extern void VmeBlockReadShort ( VmeDesc *vme, 
                         unsigned short *data, unsigned int count );

/*
 * File: atvme_mm.c
 */

__END_DECLS

#endif  /* _atvme_h */
