
/*
 * File   : atvme_mm.c
 *
 * Purpose: Libary functions for Vme bus access
 *          using the mmap() system call
 *
 * $Revision: 1.1 $
 * $Date: 1999/02/18 10:41:16 $
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <linux/atvme.h>

#include "atvme.h"
#include "vme.h"

static unsigned int Atvme_64K_page = 0xffffffff;


/* Note: Using mmap() for atvme in only possible for the
 *       D8 and D16 access modes !!!
 */

/* ------------------------------------------------------------------------- */

static void *MapNewRange ( VmeDesc *vme )
 {                                                          /* thjm 17.02.99 */
  unsigned int new_page = (unsigned int)vme->addr & 0xffff0000;
  static void *vme_page_ptr = NULL;

  if ( new_page != Atvme_64K_page )
   {
    AtvmeSet64KPage ( new_page );
    if ( !vme_page_ptr )
     {
      vme_page_ptr = AtvmeMMap();
      if ( vme_page_ptr == NULL )
        perror ( "atvme_mm.c: could not mmap()" );
    }
    
    Atvme_64K_page = new_page;
  }
  
  return vme_page_ptr;
}

/* ------------------------------------------------------------------------- */

unsigned char VmeReadChar ( VmeDesc *vme )
 {                                                          /* thjm 17.02.99 */
  unsigned char *data_ptr, data;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D8 );
  if ( ( vme->acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned char *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned char *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      return *data_ptr;
    }
    else
      return 0;
  }
  else
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

    AtvmeRead ( &data, sizeof ( unsigned char ) );
  
    return data;
  }
}

/* ------------------------------------------------------------------------- */

void VmeWriteChar ( VmeDesc *vme, unsigned char data )
 {                                                          /* thjm 17.02.99 */
  unsigned char *data_ptr;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D8 );
  if ( ( acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned char *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned char *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      *data_ptr = data;
    }
  }
  else
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

    AtvmeWrite ( &data, sizeof ( unsigned char ) );
  }
}

/* ------------------------------------------------------------------------- */

unsigned short VmeReadShort ( VmeDesc *vme )
 {                                                          /* thjm 17.02.99 */
  unsigned short *data_ptr, data;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  if ( ( vme->acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned short *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned short *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      return *data_ptr;
    }
    else
      return 0;
  }
  else
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

    AtvmeRead ( (char *)&data, sizeof ( unsigned short ) );
  
    return data;
  }
}

/* ------------------------------------------------------------------------- */

void VmeWriteShort ( VmeDesc *vme, unsigned short data )
 {                                                          /* thjm 17.02.99 */
  unsigned short *data_ptr;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  if ( ( acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned short *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned short *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      *data_ptr = data;
    }
  }
  else
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

    AtvmeWrite ( (char *)&data, sizeof ( unsigned short ) );
  }
}

/* ------------------------------------------------------------------------- */

void VmeFifoReadShort ( VmeDesc *vme, 
                         unsigned short *data, unsigned int count )
 {                                                          /* thjm 18.02.99 */
  unsigned short *data_ptr;
  int i, acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  if ( ( acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned short *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned short *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      for ( i=0; i<count; i++)
        *data++ = *data_ptr;
    }
  }
  else  /* D32 mode */
   {
    if ( !Atvme_fpos_tied )                 /* lseek() is only once needed ! */
     {
      AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
      AtvmeRead ( (char *)data, count * sizeof ( unsigned short ) );
    }
    else
     {
      for ( i=0; i<count; i++ )
       {
        AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
        AtvmeRead ( (char *)&data[i], sizeof ( unsigned short ) );
      }
    }
  }
}

/* ------------------------------------------------------------------------- */

void VmeBlockReadShort ( VmeDesc *vme, 
                         unsigned short *data, unsigned int count )
 {                                                          /* thjm 18.02.99 */
  unsigned short *data_ptr;
  int i, acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  if ( ( acc & ATVME_D_MASK ) != VME_D32 )
   {
    data_ptr = (unsigned short *)MapNewRange ( vme );
    if ( data_ptr != NULL )
     {
      data_ptr = (unsigned short *)
                 ((unsigned long)data_ptr + ((unsigned long)vme->addr & 0xffff));
      for ( i=0; i<count; i++)
        *data++ = *data_ptr++;
    }
  }
  else  /* D32 mode */
   {
    if ( !Atvme_fpos_tied )                   /* lseek() needed every time ! */
     {
      for ( i=0; i<count; i++ )
       {
        AtvmeSeek ( (unsigned int)vme->addr+2*i, SEEK_SET );
        AtvmeRead ( (char *)&data[i], sizeof ( unsigned short ) );
      }
    }
    else
     {
      AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
      AtvmeRead ( (char *)data, count * sizeof ( unsigned short ) );
    }
  }
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
