
/*
 * File   : atvme_simple.c
 *
 * Purpose: Library functions for VME bus data accesses
 *          without using the mmap() system call !!!
 *
 * $Revision: 1.5 $
 * $Date: 1999/02/12 15:46:12 $
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <linux/atvme.h>

#include "atvme.h"
#include "vme.h"

/* ------------------------------------------------------------------------- */
/* ---                   A16D8, A16D16 and A16D32 accesses               --- */
/* ------------------------------------------------------------------------- */

char AtvmeReadByteShort ( unsigned short adr )
                                              /* Get byte from short IO page */
 {
  unsigned char data;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | ATVME_A16 | ATVME_D8 );
  AtvmeSeek ( (unsigned int)adr, SEEK_SET );

  AtvmeRead ( &data, sizeof ( unsigned char ) );
  
  return ( data );
}

/* ------------------------------------------------------------------------- */

void AtvmeWriteByteShort ( unsigned short adr, unsigned char data )
                                              /* Put Byte into short IO page */
 {
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | ATVME_A16 | ATVME_D8 );
  AtvmeSeek ( (unsigned int)adr, SEEK_SET );

  AtvmeWrite ( &data, sizeof ( unsigned char ) );
}

/* ------------------------------------------------------------------------- */

int AtvmeReadWordShort ( unsigned short adr)
                                              /* Get word from short IO page */
 {
  unsigned short data;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | ATVME_A16 | ATVME_D8 );
  AtvmeSeek ( (unsigned int)adr, SEEK_SET );

  AtvmeRead ( (char *)&data, sizeof ( unsigned short ) );
  
  return ( data );
}

/* ------------------------------------------------------------------------- */

void AtvmeWriteWordShort ( unsigned short adr, unsigned short data )
                                              /* put word into short IO page */
 {
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | ATVME_A16 | ATVME_D16 );
  AtvmeSeek ( (unsigned int)adr, SEEK_SET );

  AtvmeWrite ( (char *)&data, sizeof ( unsigned short ) );
}

/* ------------------------------------------------------------------------- */

#if 0
#ifdef ATVME_HE6

int AtvmeReadLongShort ( unsigned short adr )
                                           /* get intword from short IO page */
 {
  int x;

  AtvmeSelectCsr();
  *AtvmeCSR = AtvmeCsrSave  & ATVME_ENABLE_LWORD;          /* enable intword */

  AtvmeSelectVme();
  AtvmeBuildAddress ( adr );
  x = *((unsigned short *)AtvmeMemPtr) & LOMASK;
  
  AtvmeSelectCsr();
  x |= (int)(((long)*AtvmeADR << 16) & HIMASK);         /* get upper  16 bit */
  *AtvmeCSR = AtvmeCsrSave | ATVME_DISABLE_LWORD;         /* disable intword */

  return ( x );
}

/* ------------------------------------------------------------------------- */

void AtvmeWriteLongShort ( unsigned short adr, int Data )
                                           /* put intword into short IO page */
 {
  AtvmeSelectCsr();
  *AtvmeCSR = AtvmeCsrSave  & ATVME_ENABLE_LWORD;          /* Enable intword */
  *AtvmeDR = (unsigned short) (Data >> 16);              /* put upper 16 bit */

  AtvmeSelectVme();
  AtvmeBuildAddress ( adr );
  pokew(AtvmeMemBase, adr & LOMASK, (unsigned short) Data & LOMASK);

  AtvmeSelectCsr();
  *AtvmeCSR = AtvmeCsrSave;                               /* Disable intword */
}

#endif  /* HE6 */
#endif

/* ------------------------------------------------------------------------- */
/* ---                   A24D8, A24D16 and A24D32 accesses               --- */
/* ------------------------------------------------------------------------- */

/* ------------------------------------------------------------------------- */
/* ---            functions like VmeRead...()/VmeWrite...()              --- */
/* ------------------------------------------------------------------------- */

unsigned char VmeReadChar ( VmeDesc *vme )
 {                                                          /* thjm 13.10.98 */
  unsigned char data;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D8 );
  AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

  AtvmeRead ( &data, sizeof ( unsigned char ) );
  
  return ( data );
}

/* ------------------------------------------------------------------------- */

void VmeWriteChar ( VmeDesc *vme, unsigned char data )
 {                                                          /* thjm 13.10.98 */
  int acc = vme->acc & ATVME_A_MASK;

  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D8 );
  AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

  AtvmeWrite ( &data, sizeof ( unsigned char ) );
}

/* ------------------------------------------------------------------------- */

unsigned short VmeReadShort ( VmeDesc *vme )
 {                                                          /* thjm 13.10.98 */
  unsigned short data;
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

  AtvmeRead ( (char *)&data, sizeof ( unsigned short ) );
  
  return ( data );
}

/* ------------------------------------------------------------------------- */

void VmeWriteShort ( VmeDesc *vme, unsigned short data )
 {                                                          /* thjm 13.10.98 */
  int acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );
  AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );

  AtvmeWrite ( (char *)&data, sizeof ( unsigned short ) );
}

/* ------------------------------------------------------------------------- */

/*
 * Note: The functions VmeFifoRead..() and VmeBlockRead..() are
 *       intended to read multiple data from Vme bus, but without
 *       using special BLT mode ...
 *       Therefor VmeFifoRead..() reads from a constant address
 *       whereas VmeBlockRead..() does an automatic address increment
 *       after every read() operation.
 */
 
void VmeFifoReadShort ( VmeDesc *vme, 
                        unsigned short *data, unsigned int count )
 {                                                          /* thjm 13.10.98 */
  int i, acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );

  if ( !Atvme_fpos_tied )                   /* lseek() is only once needed ! */
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
    AtvmeRead ( (char *)data, count * sizeof ( unsigned short ) );
  }
  else
   {
    for ( i=0; i<count; i++ )
     {
      AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
      AtvmeRead ( (char *)&data[i], sizeof ( unsigned short ) );
    }
  }
}

/* ------------------------------------------------------------------------- */

void VmeBlockReadShort ( VmeDesc *vme, 
                         unsigned short *data, unsigned int count )
 {                                                          /* thjm 26.01.99 */
  int i, acc = vme->acc & ATVME_A_MASK;
  
  AtvmeSetMode ( ATVME_NONPRIV | ATVME_DATA | acc | ATVME_D16 );

  if ( !Atvme_fpos_tied )                   /* lseek() needed every time ! */
   {
    for ( i=0; i<count; i++ )
     {
      AtvmeSeek ( (unsigned int)vme->addr+2*i, SEEK_SET );
      AtvmeRead ( (char *)&data[i], sizeof ( unsigned short ) );
    }
  }
  else
   {
    AtvmeSeek ( (unsigned int)vme->addr, SEEK_SET );
    AtvmeRead ( (char *)data, count * sizeof ( unsigned short ) );
  }
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
