
/*
 * File   : atvme.h
 *
 * Purpose: defines for AT_VME Interface atvme.h
 *
 * pro_VME Industrielle Rechnertechnik GmbH
 * K.Hitschler, 1.9.89
 * 
 * last  changes: K.Hitschler   17.12.90        REL2_0
 *
 * $Revision: 0.13 $
 * $Date: 1999/02/02 15:53:41 $
 */

#ifndef _linux_atvme_h_
#define _linux_atvme_h_

#include <linux/vme.h>
#include <linux/signal.h>

/*
 * Contants of board installation
 */

#define ATVME_CSR    0x0d0000    /* VME or Command/Status/Register base addr */
#define ATVME_SIZE   0x010000    /* window size 64K */
#define ATVME_IO     0x200       /* AT- IO CSR switch Address              */
#define ATVME_IO_LENGTH 4        /* four addresses are used */
#define ATVME_LEVEL  0x5         /* Interrupt level on 5 (related to Hardware) */
#define ATVME_MAILB  0xfc00L     /* Burned mailbox address in short IO */
#define ATVME_LWORD  0           /* No 32 bit VMEbus expected */

#define ATVME_IRQ_LINE	0xf      /* PC interrupt line used by the interface */


#include <errno.h>


/*
 * definitions and external declarations
 */
 
#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE  1
#endif

#define INIT_FAIL -EIO
#define INIT_SUCCESS 0

#ifndef FAIL
#define FAIL 0
#endif

#ifndef SUCCESS
#define SUCCESS 1
#endif

#ifndef NUMBER
#define NUMBER(a) sizeof(a) / sizeof(*a)
#endif

#define ATVME_ATVME_HE6       /* HE6 or HE3 itvme board format */
#define ATVME_REL2_0          /* actual hardware release or REL1.0 */

#define ATVME_MAJOR 101
#ifdef ATVME_REL2_0 
#  define ATVME_NAME "ProVme ITVME6 PC-Vme interface Rel 2.0"
#else
#  define ATVME_NAME "ProVme ITVME6 PC-Vme interface Rel 1.x"
#endif

#ifndef NEW_GFP_ATOMIC
#define NEW_GFP_ATOMIC(type) (type *)kmalloc(sizeof(type),GFP_ATOMIC)
#endif

#ifndef NEW_GFP_KERNEL
#define NEW_GFP_KERNEL(type) (type *)kmalloc(sizeof(type),GFP_KERNEL)
#endif

/*
 * ioctl() command codes ...
 */
 
#define ATVME_IOCTL_BASE                       0x00000100

#define ATVME_INITIALIZE                       (ATVME_IOCTL_BASE + 0x00)
#define ATVME_SYSRESET                         (ATVME_IOCTL_BASE + 0x01)
#define ATVME_CHECK_CONNECTION                 (ATVME_IOCTL_BASE + 0x02)

#define ATVME_SET_AM                           (ATVME_IOCTL_BASE + 0x10)
#define ATVME_ENABLE_INTERRUPT                 (ATVME_IOCTL_BASE + 0x11)
#define ATVME_DISABLE_INTERRUPT                (ATVME_IOCTL_BASE + 0x12)
#define ATVME_POST_INTERRUPT                   (ATVME_IOCTL_BASE + 0x13)
#define ATVME_REMOVE_INTERRUPT                 (ATVME_IOCTL_BASE + 0x14)
#define ATVME_GET_INTERRUPT                    (ATVME_IOCTL_BASE + 0x15)
#define ATVME_SET_64K_PAGE                     (ATVME_IOCTL_BASE + 0x16)
#define ATVME_ERROR_CHECK                      (ATVME_IOCTL_BASE + 0x17)
#define ATVME_TIE_FPOS                         (ATVME_IOCTL_BASE + 0x18)
#define ATVME_UNTIE_FPOS                       (ATVME_IOCTL_BASE + 0x19)
#define ATVME_ATTACH_SIGNAL                    (ATVME_IOCTL_BASE + 0x1A)

#define ATVME_GET_DRIVER_INFO                  (ATVME_IOCTL_BASE + 0x20)

#define ATVME_MAX_INTS_QUEUED    50

/*
 * definitions of general constants
 */
 
#define ATVME_ON         1
#define ATVME_OFF        0

#define LOMASK 0x0000ffffL    /* AND mask for lower part of intword */
#define HIMASK 0xffff0000L    /* AND mask for higher part of intword */

/*
 * List of ATVME control mask, all valid for the CSR regsiter
 * All ATVME controls are L-active
 */

                               /* Interrupt enable ATVME controls */
#define ATVME_ENABLE_VME_INT  0xfffd /* AND-Mask for VME Interrupts enable */
#define ATVME_ENABLE_BERR_INT 0xfffb /*    "     for VME Bus Error Interrupt enable */

                               /* Timeout and Bus Error ATVME controls */
#define ATVME_ENABLE_ERROR    0xfff7 /* AND-Mask for Timeout Error enable */
#define ATVME_CLEAR_ERROR     0xffff /* AND-Mask for Timeout Error disable/clear */

                               /* miscellaneaous ATVME_CONTROLS */
#define ATVME_ENABLE_LWORD    0xfffe /* AND-Mask for enable next transfer -> Lword */
#define ATVME_READ_INT_VECTOR 0xffef /* AND-Mask for enable next transfer -> Iack  */
#define ATVME_HOLD_BUS        0xff7f /* AND-Mask for VMEbus Hold */

#define ATVME_ENABLE_SYSRES_INT   0xffbf /* AND-Mask for VME-Sysreset Interr. Enable  */

/*
 * Controls for auxillary CSRegister  (Hardware REV2.0 only)
 * All contents of the auxCSR are high active !!
 */

#define ATVME_SYSRES_SET       0x0001 /* OR-Mask for VME-Sysreset generation */
                                      /* AND-Mask for VME-Sysreset clear     */
#define ATVME_SYSRES_CLEAR     ((~ATVME_SYSRES_SET) & 0xffff)
#define ATVME_SYSFAIL_SET      0x0002 /* OR-Mask for VME-Sysfail set         */
                                      /* AND-Mask for own VME-Sysfail clear  */
#define ATVME_SYSFAIL_CLEAR    ((~ATVME_SYSFAIL_SET) & 0xffff)
#define ATVME_SYSRES_INT_SET   0x0004 /* OR-Mask for set of VME-Sysreset-Int */
                                      /* AND-Mask for clear of VME-Sysreset  */
#define ATVME_SYSRES_INT_CLEAR ((~ATVME_SYSRES_INT_SET) & 0xffff)

/*
 * List of STATUS masks, all valid for CSR register
 * All STATUS bits are L-active, except the MAILBOX Contents
 */

                               /* Interrupt disable STATUS */
#define ATVME_VME_INT   0x0002 /* OR-Mask for VME Interrupts enable */
#define ATVME_BERR_INT  0x0004 /*    "    for VME Bus Error Interrupt enable */

                               /* Timeout and Bus Error STATUS */
#define ATVME_DISABLE_ERROR    0x0008 /* OR -Mask for Timeout Error disable */

                               /* miscellaneaous STATUS */
#define ATVME_DISABLE_LWORD    0x0001 /* OR-Mask for disable next transfer -> Lword */
#define ATVME_DISABLE_IACK     0x0010 /* OR-Mask for disable next transfer -> Iack  */

/*
 * Interrupt Source STATUS Masks
 */
 
#define ATVME_VMEIRQ_INT       0x0020 /* Pending Hardware Interrupt on VME */
#define ATVME_MAILBOX_INT      0x0040 /* Pending Mailbox Interrupt */
#define ATVME_ERROR_INT        0x0080 /* Pending Timeout or Bus Error Interrupt */
#define ATVME_SYSRESET_INT     0x2000 /* Mailbox-SYSRESET Interrupt mask */

#define ATVME_MAILBOX          0xff00 /* AND-Mask for Mailbox Contents, H-active */

#define ATVME_DISABLE_SYSRESET_INT   0x0040 /* AND-Mask for disable VME-Sysreset-Inter.  */

/*
 * Status masks applies to auxillary CSRegister (only REV2.0 hardware)
 * high active status for auxCSR
 */
 
#define ATVME_CHECK_SYSFAIL_INT      0x0002 /* AND-Mask for VME-Sysfail check            */
#define ATVME_CHECK_SYSRESET_INT     0x0004 /* AND-Mask for VME-Sysreset-Interrupt check */


/*
 * Some defaults, useful when working with ATVME
 */
 
#define ATVME_AM_DEFAULT         AM_SHORT_NONPRIV      /* default am code */
#define ATVME_STATUS_DEFAULT     ATVME_ENABLE_ERROR      /* default status */
#define ATVME_AUX_STATUS_DEFAULT (ATVME_SYSFAIL_CLEAR & \
				 ATVME_SYSRES_INT_CLEAR & \
				 ATVME_SYSRES_CLEAR) /* default Aux status */
#define ATVME_HI_ADR_DEFAULT     0xffff       /* default high address */
#define ATVME_HI_DATA_DEFAULT    0            /* default for high data */

#define ATVME_PRIV_MASK          0x0300
#define ATVME_SUP                0x0100
#define ATVME_NONPRIV            0x0200

#define ATVME_ACC_MASK           0x3000
#define ATVME_BLOCK              0x1000
#define ATVME_PROG               0x2000
#define ATVME_DATA               0x3000

#define ATVME_A_MASK             0x07
#define ATVME_A16                0x01
#define ATVME_A24                0x02
#define ATVME_A32                0x03
#define ATVME_A64                0x04

#define ATVME_D_MASK             0x30
#define ATVME_D8                 0x10
#define ATVME_D16                0x20
#define ATVME_D32                0x30


typedef struct
 {
  unsigned short addr_type;
  unsigned short data_type;
  unsigned short acc_type;

} AtvmeAD;


typedef struct
 {
  AtvmeAD        ad;
  unsigned short msw_addr;
  unsigned short am;

} AtvmeState;


typedef enum { AtvmeRoak, AtvmeRoraByUser, AtvmeRoraByHandler } AtvmeAckType;


typedef struct
 {
  int            int_vec;
  unsigned long  int_tl;
  AtvmeAckType   ack_type;
  unsigned int   rora_reg;
  unsigned int   acc_mask;
  
} IntAckStruct;

#endif  /* _linux_atvme_h_ */
