#include <linux/list.h>

Node *newNode(void)
{
  Node *mynode;
      
  mynode = NEW_GFP_ATOMIC(Node);
  if (!mynode) return NULL;
  mynode->pred = NULL;
  mynode->succ = NULL;
  mynode->data = NULL;
   
  return mynode;
}

Node *addNode(Node *pos, void *insert, enum NodePosition p)
{
  Node *mynode;
  mynode = NEW_GFP_ATOMIC(Node);
  if (!mynode) return NULL;
  if (p == CurrentLeft) {
    mynode->pred = pos;
    mynode->succ = pos->succ;
    pos->succ = mynode;
  } else {
    mynode->pred = pos->pred;
    pos->pred = mynode;
    mynode->succ = pos;
  }
  mynode->data = insert;
  return mynode;
}

Node *delNodeAndData(Node *pos)
{
  Node *tail = NULL;
  if (!pos->pred) return NULL;
  /* can't delete head */
  if (!pos->succ) {
    tail = pos->pred;
    tail->succ = NULL;
  } else {
    pos->pred->succ = pos->succ;
    pos->succ->pred = pos->pred;
  }
  free(pos->data);
  free(pos);
  return tail;
}

Node *delNode(Node *pos)
{
  Node *tail = NULL;
  if (!pos->pred) return NULL;
  /* can't delete head */
  if (!pos->succ) {
    tail = pos->pred;
    tail->succ = NULL;
  } else {
    pos->pred->succ = pos->succ;
    pos->succ->pred = pos->pred;
  }
  free(pos);
  return tail;
}
/* ------------------------------ */

List *newList(void)
{
  List *ret = NEW_GFP_KERNEL(List);
  if (!ret) return NULL;
  ret->head = ret->tail = newNode();
  ret->nodes = 0;
  return ret;
}

void deleteList(List *l)
{
  if (l) {
    resetList(l);
    free(l->head);
    free(l);
  }
}

Node *addTail(List *list, void *insert)
{
  if (!list) return NULL;
  list->tail = addNode(list->tail,insert,CurrentLeft);
  list->nodes++;
  return list->tail;
}

Node *addHead(List *list, void *insert)
{
  Node *node;
  if (!list) return NULL;
  if (!list->head->succ) {
    return addTail(list,insert);
  } else {
    node = addNode(list->head,insert,CurrentRight);
  }
  list->nodes++;
  return node;
}

Node *setCurr(List *list, void *c, int (*comp)(void *, void *))
{
  Node *n; 

  if (!list) return NULL;
  n = list->head->succ;
  while (n) {
    if (!comp(c,n->data)) {
      return n;
    } 
    n = n->succ;
  }  
  return NULL;
}

void delNodeAndDataInList(List *l, Node *pos)
{
  Node *n;
  if (!l) return;
  if ((n = delNodeAndData(pos))) l->tail = n;
  l->nodes--;
}

void delNodeInList(List *l, Node *pos)
{
  Node *n;
  if (!l) return;
  if ((n = delNode(pos))) l->tail = n;
  l->nodes--;
}

int emptyList(List *l)
{
  if (!l) return 1;
  return l->head->succ ? 0 : 1;
}

void delTailAndData(List *l)
{
  Node *t;
  if (!l) return;
  t = l->tail;
  if (emptyList(l)) return;
  l->tail = l->tail->pred;
  l->tail->succ = NULL;
  free(t->data);
  free(t);
}

void delTail(List *l)
{
  Node *t;
  if (!l) return;
  t = l->tail;
  if (emptyList(l)) return;
  l->tail = l->tail->pred;
  l->tail->succ = NULL;
  free(t);
}

void resetList(List *l)
{
  Node *n;

  if (!l) return;
  n = l->tail;
  while (n->pred) {
    n = delNodeAndData(n);
  }
  l->tail = l->head;
  l->nodes = 0;
}

Node *getFirstNode(List *l)
{
  if (!l) return NULL;
  return l->head->succ;
}

Node *getLastNode(List *l)
{
  if (!l) return NULL;
  return l->tail;
}

void delFirstNode(List *l)
{
  if (!l) return;
  delNodeInList(l,getFirstNode(l));
}

void delFirstNodeAndData(List *l)
{
  if (!l) return;
  delNodeAndDataInList(l,getFirstNode(l));
}

void delLastNode(List *l)
{
  if (!l) return;
  delNodeInList(l,getLastNode(l));
}

void delLastNodeAndData(List *l)
{
  if (!l) return;
  delNodeAndDataInList(l,getLastNode(l));
}

int getNumOfNodesInList(List *l)
{
  int ret;
  if (!l) return 0;
  ret = l->nodes;
  return ret;
}

void printList(List *l, char *(*StructToString)(void *))
{
  if (!l) return;
#ifdef I_HAVE_ISERVER
  Node *n = l->head;
  int i=1;
  if (!n->succ) {
    printf("List empty!\n");
    return;
  }
  while (n->succ) {
    printf("Node %d: %s",i++,StructToString(n->succ->data));
    printf("\n");
    n = n->succ;
  }
#else
  l = NULL;
  StructToString = NULL;
#endif
}
